package com.github.polpetta.mezzotre.helper;

import com.google.common.io.Resources;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import io.ebean.Database;
import io.ebean.DatabaseFactory;
import io.ebean.config.DatabaseConfig;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;
import org.apache.commons.lang3.tuple.Pair;
import org.testcontainers.containers.PostgreSQLContainer;

public class Loader {

  public static Pair<Properties, Properties> loadDefaultEbeanConfigWithPostgresSettings(
      PostgreSQLContainer<?> container) throws IOException {
    final URL ebeanRes = Resources.getResource("database-test.properties");
    final URL hikariRes = Resources.getResource("hikari.properties");
    final Properties ebeanConnectionProperties = new Properties();
    final Properties hikariConnectionProperties = new Properties();
    try (InputStream ebeanInputStream = ebeanRes.openStream();
        InputStream hikariInputStream = hikariRes.openStream()) {
      hikariConnectionProperties.load(hikariInputStream);
      hikariConnectionProperties.put("username", container.getUsername());
      hikariConnectionProperties.put("password", container.getPassword());
      hikariConnectionProperties.put("jdbcUrl", container.getJdbcUrl());

      ebeanConnectionProperties.load(ebeanInputStream);
      ebeanConnectionProperties.put("datasource_db_username", container.getUsername());
      ebeanConnectionProperties.put("datasource_db_password", container.getPassword());
      ebeanConnectionProperties.put("datasource_db_databaseUrl", container.getJdbcUrl());
      ebeanConnectionProperties.put("datasource_db_databaseDriver", "org.postgresql.Driver");
    }
    return Pair.of(ebeanConnectionProperties, hikariConnectionProperties);
  }

  public static Database connectToDatabase(Properties ebean, Properties hikari) {
    final HikariDataSource hikariDataSource = new HikariDataSource(new HikariConfig(hikari));
    final DatabaseConfig databaseConfig = new DatabaseConfig();
    databaseConfig.loadFromProperties(ebean);
    databaseConfig.setDataSource(hikariDataSource);

    return DatabaseFactory.create(databaseConfig);
  }

  public static Database connectToDatabase(Pair<Properties, Properties> connectionProperties) {
    return connectToDatabase(connectionProperties.getLeft(), connectionProperties.getRight());
  }
}
