package com.github.polpetta.mezzotre.orm.model;

import static org.junit.jupiter.api.Assertions.*;

import com.github.polpetta.mezzotre.helper.Loader;
import com.github.polpetta.mezzotre.helper.TestConfig;
import com.github.polpetta.mezzotre.orm.model.query.QUser;
import com.github.polpetta.mezzotre.util.Clock;
import io.ebean.Database;
import io.ebean.SqlRow;
import java.sql.Timestamp;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.testcontainers.containers.PostgreSQLContainer;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

@Tag("slow")
@Tag("database")
@Testcontainers
public class UserIntegrationTest {

  @Container
  private final PostgreSQLContainer<?> postgresServer =
      new PostgreSQLContainer<>(TestConfig.POSTGRES_DOCKER_IMAGE);

  private Database database;

  @BeforeEach
  void setUp() throws Exception {
    database =
        Loader.connectToDatabase(Loader.loadDefaultEbeanConfigWithPostgresSettings(postgresServer));
  }

  @Test
  void shouldInsertEntryIntoDatabase() {

    final User user = new User("1234", "example@example.com", true, null);

    user.save();
    final String query = "select * from registered_user where id = ?";
    final SqlRow savedUser = database.sqlQuery(query).setParameter("1234").findOne();

    assertNotNull(savedUser);
    assertEquals("1234", savedUser.getString("id"));
  }

  @Test
  void shouldRetrieveEntryInDatabase() {

    final Timestamp timestampFromUnixEpoch = Clock.getTimestampFromUnixEpoch(1L);

    final String insertQuery =
        "insert into registered_user (id, is_active, telegram_id, email_address, entry_created,"
            + " entry_modified) values (?, ?, null, ?, ?, ?)";
    final int affectedRows =
        database
            .sqlUpdate(insertQuery)
            .setParameter("id123")
            .setParameter(true)
            .setParameter("example@example.com")
            .setParameter(timestampFromUnixEpoch)
            .setParameter(timestampFromUnixEpoch)
            .execute();

    assertEquals(1, affectedRows);

    final User id123 = new QUser().id.eq("id123").findOne();
    assertNotNull(id123);
    assertTrue(id123.isActive());
    assertNull(id123.getTelegramId());
  }
}
